%define releasebase 1
%define nbase fftw
%define prefix /opt/fft/%{nbase}/%{version}/%{compilertag}
%define modulepath /opt/Modules/modulefiles/
#Use as deafults if no command line inputs
%define as_limulus 1
%define use_gnu4  1
# allow --with[out] <feature> at rpm command line build
%{?_with_gnu3: %{expand: %%define use_gnu3 1}}
%{?_with_gnu4: %{expand: %%define use_gnu4 1}}
%{?_with_pgi: %{expand: %%define use_pgi 1}}
%{?_with_intel: %{expand: %%define use_intel 1}}
%{?_with_pathsc: %{expand: %%define use_pathsc 1}}
%{?_with_basement: %{expand: %%define as_basement 1}}
%{?_with_limulus: %{expand: %%define as_limulus 1}}
#pick the target
%if "1" == "%{as_basement}"
%define targetbase Basement
%else
%define targetbase Limulus
%endif
%global _compiler_load  module load %{compilertag}
%global _compiler_unload  module unload %{compilertag}

# pick the compiler
%if "1" == "%{use_gnu3}"
%define compilertag gnu3
%define openMP  %{nil}
%define sse2 --enable-sse2
%define sse --enable-sse
define setenvs export CC=gcc; export CFLAGS='--enable-threads -O2 -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'; export F77=g77; export FFLAGS='--enable-threads -O2'
%else
%if "1" == "%{use_pgi}"
%define compilertag pgi
%define openMP  --enable-openmp
%define sse2 --enable-sse2
%define sse --enable-sse
%define setenvs export CC=pgcc; export F77=pgf77; export CXX=pgCC; export OPENMP_CFLAGS='-mp'; export CFLAGS=' -O2'; export CXXLAGS=' -O2 '; export FFLAGS=' -O2'
%else
%if "1" == "%{use_intel}"
%define compilertag intel
%define openMP --enable-openmp 
%define sse2 --enable-sse2
%define sse --enable-sse
%define setenvs export F77=ifort; export FFLAGS='-O2 ';export CC=icc; export CFLAGS='-O2';export OPENMP_CFLAGS='-openmp';export CXX=icpc;
%else
%if "1" == "%{use_pathsc}"
%define compilertag pathsc
echo "Path Comp not implemented"
exit
%else
%define compilertag gnu4
%define openMP --enable-openmp 
%define sse2 --enable-sse2
%define sse --enable-sse
%define avx --enable-avx
%define setenvs export CC=gcc; export CFLAGS='-O2 -fPIC -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'; export F77=gfortran; export FFLAGS=' -O2 -fPIC'; export OPENMP_CFLAGS='-fopenmp'
%endif
%endif
%endif
%endif

Name: %{nbase}-%{compilertag}

Summary: Fast Fourier Transform library
Version: 3.3.4
Group: %{targetbase}/Libraries
Release: %{releasebase}.%{targetbase}
License: GPL
URL: http://www.fftw.org/
Packager: deadline@basement-supercomputing.com
Vendor: Basement-Supercomputing.com
Source: %{nbase}-%{version}.tar.gz
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
Requires: modules
BuildRequires: modules
Provides:%{nbase}-%{compilertag}=%{version}

%description
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package also contains the header files, static libraries and development
documentation for %{name}. It has been configured to work in
the Basement Supercomputer Cluster environment. In this case only static libaries
are included.

%prep
%setup -q -c  %{nbase}-%{version}
mv %{nbase}-%{version} single
cp -a single double
cp -a single long

%build
rpm -qf /opt/Modules/bin/modulecmd >& /dev/null
if [ $? -ne 0 ]; then
  echo "This RPM requries the BSC Modules package to build"
fi
. /etc/profile.d/modules.sh
# there is no module for gnu4, gnu3, these are default
# the module command just fails to load these modules
# and continues with the gnu compilers, a bit ugly but it works
%{_compiler_load}

%setenvs

### build double
pushd double
./configure --prefix=%{prefix} %{openMP}  %{sse2} %{avx} 
%{__make} %{?_smp_mflags}
popd
pushd single
### build single
./configure --prefix=%{prefix}   %{openMP} %{sse} %{avx} \
	--enable-float 
%{__make} %{?_smp_mflags}
popd
pushd long
### build long
./configure --prefix=%{prefix}   %{openMP}    \
        --enable-long-double 
%{__make} %{?_smp_mflags}
popd
%{_compiler_unload}


%install
pushd double
        make  DESTDIR=%{buildroot} install 
        cp -a AUTHORS COPYING COPYRIGHT ChangeLog NEWS README* TODO ../
        cp -a doc/ ../
popd
pushd single
        make  DESTDIR=%{buildroot} install 
popd
pushd long
        make  DESTDIR=%{buildroot} install 
popd


# remove package config information
/bin/rm -r  %{buildroot}/%{prefix}/lib/pkgconfig

# create /opt//etc/ld.so.conf.d/
echo "Creating /opt//etc/ld.so.conf.d/%{name} file"
mkdir -p %{buildroot}/opt/etc/ld.so.conf.d
cat <<EOF> %{buildroot}/opt/etc/ld.so.conf.d/%{name}-%{version}.conf
%{prefix}/lib
EOF

#create the module file
mkdir -p  %{buildroot}/%{modulepath}/%{nbase}/%{version}
cat <<EOF > %{buildroot}/%{modulepath}/%{nbase}/%{version}/%{compilertag}
#%Module1.0#######################################################################
## %{nbase} modulefile
##

proc ModulesHelp { } {
        puts stderr "\tThis module loads the appropriate environment for "
        puts stderr "\tusing %{name}  version %{version}.\n"
}

module-whatis "%{name} %{version}"
module-log error none

set fftw3root %{prefix}

if [ module-info mode load ] {
  prepend-path  LD_LIBRARY_PATH \$fftw3root/lib
	setenv FFTW3_PATH \$fftw3root/lib
	setenv FFTW3_INCLUDE \$fftw3root/include
  append-path MANPATH \$fftw3root/share/man
  append-path INFOPATH \$fftw3root/share/info
}
if [ module-info mode remove ] {
  remove-path LD_LIBRARY_PATH \$fftw3root/lib
	unsetenv FFTW3_PATH 
	unsetenv FFTW3_INCLUDE 
  remove-path MANPATH \$fftw3root/share/man
  remove-path INFOPATH \$fftw3root/share/info
}
EOF

%post
/sbin/ldconfig 2>/dev/null

%postun
/sbin/ldconfig 2>/dev/null

%clean
%{__rm} -rf %{buildroot}

%files
%defattr(-, root, root, 0755)
%doc AUTHORS COPYING COPYRIGHT ChangeLog NEWS README* TODO doc/*.png doc/html doc/*.pdf doc/FAQ/fftw-faq.html/
%{prefix}/include/*.h
%{prefix}/include/*.f03
%{prefix}/include/*.f
%{prefix}/bin/*
%{prefix}/share/*
%{prefix}/lib/*.a
%{prefix}/lib/*.la
%{modulepath}/%{nbase}/%{version}/%{compilertag}
/opt/%{_sysconfdir}/ld.so.conf.d/%{name}-%{version}.conf


%changelog
* Fri Dec 12 2012  Douglas Eadline <deadline@basement-supercomputing.com>
- added /opt/etc/ld.so.conf.d entry
* Fri Oct 05 2012 - Douglas Eadline <deadline@basement-supercomputing.com>
- first release

